package lib

import (
	"fmt"
	"os"
	"path/filepath"
)

type Config struct {
	NatsAddr string
	Project  string
	Module   string

	ParamList bool
	ParamSet  bool
	ParamGet  bool
	ParamFile bool

	ParamName  string
	ParamValue string

	Filename string
	Dirname  string

	GatewayFuncSet      bool
	GatewayProxySet     bool
	GatewayStaticSet    bool
	GatewayStaticDirSet bool

	GatewayPath      string
	Domain           string
	GatewayFuncName  string
	GatewayProxyAddr string

	Method string

	VerifyAccessToken     bool
	VerifyAccessTokenPath string

	CallFunction string // Вызываемая функция
	Payload      []byte // Бинарные данные
}

var AppConfig Config

func LoadConfig() {

	var exists bool

	if AppConfig.NatsAddr, exists = os.LookupEnv("NATS_ADDR"); !exists {
		AppConfig.NatsAddr = "nats://localhost:4200"
	}
	if AppConfig.Project, exists = os.LookupEnv("PROJECT"); !exists {
		AppConfig.Project = "system"
	}
	if AppConfig.Module, exists = os.LookupEnv("MODULE"); !exists {
		AppConfig.Module = "cli"
	}

	parseArgs(os.Args)

}

func parseArgs(args []string) {
	if len(args) > 0 {
		switch args[0] {
		case "--help":
			PrintHelp()
			os.Exit(0)
		case "--list":
			AppConfig.ParamList = true
			parseArgs(args[1:])
		case "--set":
			if len(args) > 2 {
				AppConfig.ParamSet = true
				AppConfig.ParamName = args[1]
				AppConfig.ParamValue = args[2]
				parseArgs(args[3:])
			} else {
				panic(fmt.Errorf("error count arguments for setting parameter"))
			}
		case "--get":
			if len(args) > 1 {
				AppConfig.ParamGet = true
				AppConfig.ParamName = args[1]
				parseArgs(args[2:])
			}
		case "--domain":
			if len(args) > 1 {
				AppConfig.Domain = args[1]
				parseArgs(args[2:])
			} else {
				panic(fmt.Errorf("error count arguments for domain"))
			}
		case "--function":
			if len(args) > 2 {
				AppConfig.GatewayFuncSet = true
				AppConfig.GatewayPath = args[1]
				AppConfig.GatewayFuncName = args[2]
				parseArgs(args[3:])
			} else {
				panic(fmt.Errorf("array count arguments for gateway function"))
			}
		case "--method":
			if len(args) > 1 {
				AppConfig.Method = args[1]
				parseArgs(args[2:])
			} else {
				panic(fmt.Errorf("array count arguments for gateway function"))
			}
		case "--proxy":
			if len(args) > 2 {
				AppConfig.GatewayProxySet = true
				AppConfig.GatewayPath = args[1]
				AppConfig.GatewayProxyAddr = args[2]
				parseArgs(args[3:])
			} else {
				panic(fmt.Errorf("array count arguments for gateway function"))
			}
		case "--static-file":
			if len(args) > 2 {
				AppConfig.GatewayStaticSet = true
				AppConfig.GatewayPath = args[1]
				AppConfig.Filename = args[2]
				parseArgs(args[3:])
			} else {
				panic(fmt.Errorf("array count arguments for gateway function"))
			}
		case "--static-dir":
			if len(args) > 2 {
				AppConfig.GatewayStaticDirSet = true
				AppConfig.GatewayPath = args[1]
				AppConfig.Dirname = args[2]
				parseArgs(args[3:])
			} else {
				panic(fmt.Errorf("array count arguments for gateway function"))
			}
		case "--upload-settings":
			if len(args) > 1 {
				AppConfig.ParamFile = true
				AppConfig.Filename = args[1]
				parseArgs(args[2:])
			} else {
				panic(fmt.Errorf("error count arguments for parameters upload-settings"))
			}
		case "--token":
			if len(args) > 1 {
				AppConfig.VerifyAccessToken = true
				AppConfig.VerifyAccessTokenPath = args[1]
				parseArgs(args[2:])
			} else {
				panic(fmt.Errorf("error count arguments for parameters token"))
			}
		case "--call":
			if len(args) > 1 {
				AppConfig.CallFunction = args[1]
				parseArgs(args[2:])
			} else {
				panic(fmt.Errorf("error count arguments for parameters call"))
			}
		case "--file":
			if len(args) > 1 {
				var fullpath string
				var err error
				fullpath, err = filepath.Abs(args[1])
				if err != nil {
					panic(fmt.Errorf("file not found"))
				}
				AppConfig.Payload, err = os.ReadFile(fullpath)
				if err != nil {
					panic(fmt.Errorf("file not found"))
				}
				parseArgs(args[2:])
			} else {
				panic(fmt.Errorf("error count arguments for parameters file"))
			}
		default:
			parseArgs(args[1:])
		}
	}
}

func PrintHelp() {
	fmt.Println(`Установка различных параметров для разных частей системы.
Использование:
	NATS_ADDR=nats://<addr> PROJECT=<name> MODULE=<name> ./yt-cli <args>
где:
	NATS_ADDR - адрес менеджера очередей [nats://localhost:4222]
	PROJECT - название проекта, для которого применяются параметры [system]
	MODULE - модуль проекта, для которого применяются параметры [cli]

Аргументы:
	--help - это окно
	--list - список параметров
	--set <name> <value> - Установить параметр
	--get <name> - получить значение параметра
	--domain <name> - Доменное имя
	--function <name> <path> - установить функцию для указанного роутинга
	--method <name> - метод, на который присваивается функция
	--proxy <path> <proxyAddr> - проксирование запроса на другой адрес по указанному пути
	--static-file <path> <file> - установка статичного файла по конкретному пути
	--static-dir <path> <dir> - установка статичных файлов с вложенными файлами по конкретному пути рекурсивно с вложениями
	--upload-settings <file> - файл настроек для массовой установки значений
	--token <path> - установить проверку JWT для указанного пути и его вложений
	--call <func-name> - вызвать функцию, которой передаются данные из файла в --file. Если нет параметра --file, данные не передаются
	--file <path/name>
	`)
}
